/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.jsdi;

/**
 * Abstraction of a given location in a debugging JavaScript
 * 
 * @since 1.0
 */
public interface Location {

	/**
	 * Returns the underlying {@link ScriptReference} for this {@link Location}
	 * 
	 * @return the underlying {@link ScriptReference} or <code>null</code> if there was an exception creating the location
	 */
	public ScriptReference scriptReference();

	/**
	 * Returns the line number of of this location within its underlying {@link ScriptReference}
	 * 
	 * @return the line number in the {@link ScriptReference}
	 */
	public int lineNumber();

	/**
	 * Returns the name of the function this location resides in, if any
	 * 
	 * @return the function name this location resides in or <code>null</code> if this location is not enclosed by a function
	 */
	public String functionName();
}
